%projectile
% GUI which compares projectile motion with drag force proportioal to v^2
% and drag-free motion in a constant gravitational field.
%
% LAST UPDATED by Andy French May 2016

function varargout = projectile(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @projectile_OpeningFcn, ...
    'gui_OutputFcn',  @projectile_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% Harmonograph opening function. This executes just before projectile is made visible.
function projectile_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for projectile
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;
d.plot = [];
d.plot2 = [];
d.plot_nodrag = [];
d.plot2_nodrag = [];
d.L = [];

%Center figure
figcentre( d.h.FIGUREprojectile );

%Set defaults
projectile_defaults

%Update projectile
projectile_update;

%%

% Default parameters
function projectile_defaults
global d

%Launch height h /m
d.y0 = 2;
d.hmax = 20;

%Launch velocity /ms^-1
d.u = 10;
d.umax = 100;

%Launch angle /deg
d.theta_deg = 45;
d.theta = d.theta_deg*pi/180;
d.theta_deg_max = 90;

%Projectile mass /kg
d.m = 0.01;
d.mmax = 0.3;

%Gravitational field strength g /ms^-2
d.g = 9.81;

%Cross sectional area of projectile /m^2
d.A = 0.001;

%Density of air /kgm^-2
d.rho = 1;

%Drag coefficient of projectile
d.cD = 0.1;

%Time interval /s for numeric trajectory computation method
d.dt = 0.01;

%Maximum number of computation points
d.N = 500;

%Fontsize for graphs
d.fsize = 14;

%Number of significant figures for graph title
d.sfs = 3;

%Default is that fixed axes scale is off
d.fix_scale = 0;

%Plot externally and save .PNG
d.plot_and_save_PNG = 0;

%Set GUI parameters
set( d.h.EDITh,'string',num2str(d.y0) );
set( d.h.EDITu,'string',num2str(d.u) );
set( d.h.EDITtheta,'string',num2str(d.theta_deg) );
set( d.h.EDITm,'string',num2str(d.m) );
set( d.h.EDIThmax,'string',num2str(d.hmax) );
set( d.h.EDITumax,'string',num2str(d.umax) );
set( d.h.EDITthetamax,'string',num2str(d.theta_deg_max) );
set( d.h.EDITmmax,'string',num2str(d.mmax) );
set( d.h.EDITg,'string',num2str(d.g) );
set( d.h.EDITA,'string',num2str(d.A) );
set( d.h.EDITrho,'string',num2str(d.rho) );
set( d.h.EDITcD,'string',num2str(d.cD) );
set( d.h.SLIDERh,'value',d.y0/d.hmax );
set( d.h.SLIDERu,'value',d.u/d.umax );
set( d.h.SLIDERtheta,'value',d.theta_deg/90);
set( d.h.SLIDERm,'value',d.m/d.mmax);

%%

%% Diffractor output function. Outputs from this function are returned to the command line.
function varargout = projectile_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Toggle fix axes scale for plot
function TOGGLEfixaxesscale_Callback(hObject, eventdata, handles)
global d
if d.fix_scale==1
    d.fix_scale = 0;
else
    d.fix_scale = 1;
end
projectile_update

%%

%% Save PNG of graph
function PUSHsavepng_Callback(hObject, eventdata, handles)
global d
d.plot_and_save_PNG = 1;
projectile_update;
d.plot_and_save_PNG = 0;

%%

%% Launch height slider
function SLIDERh_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERh,'value');

%Determine vale of N and update associated edit box
d.y0 = v * d.hmax;
set(d.h.EDITh,'string',num2str(d.y0));

%Update GUI
projectile_update

%%

%% Launch velocity slider
function SLIDERu_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERu,'value');

%Determine vale of u and update associated edit box
d.u = v * d.umax;
set(d.h.EDITu,'string',num2str(d.u));

%Update GUI
projectile_update

%%

%% Launch angle slider
function SLIDERtheta_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERtheta,'value');

%Determine vale of N and update associated edit box
d.theta_deg = v * 90;
d.theta = d.theta_deg*pi/180;
set(d.h.EDITtheta,'string',num2str(d.theta_deg));

%Update GUI
projectile_update

%%

%% Projectile mass slider
function SLIDERm_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERm,'value');

%Determine vale of N and update associated edit box
d.m = v * d.mmax;
set(d.h.EDITm,'string',num2str(d.m));

%Update GUI
projectile_update

%%

%% Launch height edit box
function EDITh_Callback(hObject, eventdata, handles)
global d
d.y0 = str2num( get(handles.EDITh,'string') );
set( d.h.SLIDERh,'value',d.y0/d.hmax);

%Update GUI
projectile_update

%%

%% Launch velocity edit box
function EDITu_Callback(hObject, eventdata, handles)
global d
d.u = str2num( get(handles.EDITu,'string') );
set( d.h.SLIDERu,'value',d.u/d.umax);

%Update GUI
projectile_update

%%

%% Launch angle edit box
function EDITtheta_Callback(hObject, eventdata, handles)
global d
d.theta_deg = str2num( get(handles.EDITtheta,'string') );
d.theta = d.theta_deg*pi/180;
set( d.h.SLIDERtheta,'value',d.theta_deg/90);

%Update GUI
projectile_update

%%

%% Projectile mass edit box
function EDITm_Callback(hObject, eventdata, handles)
global d
d.m = str2num( get(handles.EDITm,'string') );
set( d.h.SLIDERm,'value',d.m/d.mmax);

%Update GUI
projectile_update

%%

%% Launch height maximum edit box
function EDIThmax_Callback(hObject, eventdata, handles)
global d
d.hmax = str2num( get(handles.EDIThmax,'string') );
if d.y0>d.hmax
    d.y0 = d.hmax;
    set(d.h.EDITh,'string',num2str(d.y0) );
end
set( d.h.SLIDERh,'value',d.y0/d.hmax);

%Update GUI
projectile_update

%%

%% Launch velocity maximum edit box
function EDITumax_Callback(hObject, eventdata, handles)
global d
d.umax = str2num( get(handles.EDITu,'string') );
if d.u>d.umax
    d.u = d.umax;
    set(d.h.EDITu,'string',num2str(d.u) );
end
set( d.h.SLIDERu,'value',d.u/d.umax);

%Update GUI
projectile_update

%%

%% Launch angle maximum edit box
function EDITthetamax_Callback(hObject, eventdata, handles)
global d
d.theta_deg_max = str2num( get(handles.EDITthetamax,'string') );
if d.theta_deg >d.theta_deg_max
    d.theta_deg = d.theta_deg_max;
    d.theta = d.theta_deg*pi/180;
    set(d.h.EDITtheta,'string',num2str(d.theta_deg) );
end
set( d.h.SLIDERtheta,'value',d.theta_deg/90);

%Update GUI
projectile_update

%%

%% Projectile mass maximum edit box
function EDITmmax_Callback(hObject, eventdata, handles)
global d
d.mmax = str2num( get(handles.EDITmmax,'string') );
if d.m>d.mmax
    d.m = d.mmax;
    set(d.h.EDITm,'string',num2str(d.m) );
end
set( d.h.SLIDERm,'value',d.m/d.mmax);

%Update GUI
projectile_update

%%

%% g edit box
function EDITg_Callback(hObject, eventdata, handles)
global d
d.g = str2num( get( d.h.EDITg,'string') );
projectile_update

%%

%% A editbox
function EDITA_Callback(hObject, eventdata, handles)
global d
d.A = str2num( get( d.h.EDITA,'string') );
projectile_update

%%

%% cD edit box
function EDITcD_Callback(hObject, eventdata, handles)
global d
d.cD = str2num( get( d.h.EDITcD,'string') );
projectile_update

%%

%% rho edit box
function EDITrho_Callback(hObject, eventdata, handles)
global d
d.rho = str2num( get( d.h.EDITrho,'string') );
projectile_update

%%

%% Choose plot pop-up-menu
function POPUPMENUchoosegraph_Callback(hObject, eventdata, handles)
global d
%Force plots to be regenerated
d.plot = [];
d.plot_nodrag =[];
d.plot2 = [];
d.plot_nodrag2 =[];
d.fix_scale = 0;
set(d.h.TOGGLEfixaxesscale,'value',0);
projectile_update

%%

%Update GUI
function projectile_update
global d

%Calculate kinematic variables t,x,y,vx,vy, ax, ay

%Drag constant k such that drag force is F = k*v^2
k = 0.5*d.cD*d.rho*d.A;

%Initialise variables
n = 1;
tt = 0;
xx = 0;
yy = d.y0;
vvx = d.u*cos(d.theta);
vvy = d.u*sin(d.theta);
vv = sqrt( vvx^2 + vvy^2 );
aax = -(k/d.m)*vv*vvx;
aay = -d.g -(k/d.m)*vv*vvy;

%Initialise variable stores (vectors)
t = tt;
x = xx;
y = yy;
vx = vvx;
vy= vvy;
v = vv;
ax = aax;
ay = aay;

%Define stopping criteia
stop_condition = ( (tt>0) && (yy<0)  ) || (n>d.N);

%Step through time and compute trajectory using a numerical method
while stop_condition == 0
    
    %Update kinematic parameters using Euler's method
    tt = tt + d.dt;
    vvx = vvx + aax*d.dt;
    vvy = vvy + aay*d.dt;
    vv = sqrt( vvx^2 + vvy^2 );
    xx = xx + vvx*d.dt + 0.5*aax*(d.dt^2);
    yy = yy + vvy*d.dt + 0.5*aay*(d.dt^2);
    aax = -(k/d.m)*vv*vvx;
    aay = -d.g -(k/d.m)*vv*vvy;
    
    %Update parameter store
    t = [t,tt];
    x = [x,xx];
    y = [y,yy];
    vx = [vx,vvx];
    vy= [vy,vvy];
    v = [v,vv];
    ax = [ax,aax];
    ay = [ay,aay];
    
    %Update stopping criteia
    n = n + 1;
    stop_condition = ( (tt>0) && (yy<0)  ) || (n>d.N);
end
d.tof_drag = tt(end);

%Determine drag-free trajectory.
%Firstly find time of flight such that y = 0.
a = -0.5*d.g;
b = d.u*sin(d.theta);
c = d.y0;
t1 = ( -b + sqrt( b^2 - 4*a*c ) )/(2*a);
t2 = ( -b - sqrt( b^2 - 4*a*c ) )/(2*a);
d.tof_nodrag = max([t1,t2]);
T = linspace(0,d.tof_nodrag,d.N);
x_nodrag = d.u*cos(d.theta)*T;
y_nodrag = d.u*sin(d.theta)*T - 0.5*d.g*(T.^2) + d.y0;
vx_nodrag = d.u*cos(d.theta);
vy_nodrag = d.u*sin(d.theta) - d.g*T;
ax_nodrag = zeros(1,d.N);
ay_nodrag = -d.g*ones(1,d.N);

%Determine the bounding parabola for the given launch velocity. Use this to
%fix the axis scales
d.ymax = d.y0 + (d.u^2)/(2*d.g);
d.xmax = sqrt( ( 2*(d.u^2)/d.g )*( d.y0 + (d.u^2)/(2*d.g) ) );
xbound = linspace(0,d.xmax,d.N);
ybound = d.y0 + (d.u^2)/(2*d.g) - (d.g).*(xbound.^2)/(2*d.u^2);

%%

%Update time of flight /s and range /m edit boxes
set( d.h.EDITtofdrag,'string',num2str(d.tof_drag) );
set( d.h.EDITtofnodrag,'string',num2str(d.tof_nodrag) );
set( d.h.EDITrangedrag,'string',num2str(max(x)) );
set( d.h.EDITrangenodrag,'string',num2str(max(x_nodrag)) );

%Plot x vs y curve

%What to do if plotting to a new figure window
if d.plot_and_save_PNG==1
    fig = figure;
    figure(fig);
end

%What to do if plot is empty
if ( isempty( d.plot ) ==1 ) || (d.plot_and_save_PNG==1)
    
    %Clear axes and delete legend
    cla;
    delete(d.L);
    
    %Create plot, depending on menu option selected
    if get( d.h.POPUPMENUchoosegraph,'value' ) == 2
        %vx and vy vs t plot
        d.plot = plot( t,vx,'r-'); hold on;
        d.plot2 = plot( t,vy,'b-');
        d.plot_nodrag = plot( T,vx_nodrag,'r--');
        d.plot2_nodrag = plot( T,vy_nodrag,'b--' );
        xlabel('t/s','fontsize',d.fsize);
        ylabel('velocity /ms^{-1}','fontsize',d.fsize);
    elseif get( d.h.POPUPMENUchoosegraph,'value' ) == 3
        %ax and ay vs t plot
        d.plot = plot( t,ax,'r-'); hold on;
        d.plot2 = plot( t,ay,'b-');
        d.plot_nodrag = plot( T,ax_nodrag,'r--');
        d.plot2_nodrag = plot( T,ay_nodrag,'b--' );
        xlabel('t/s','fontsize',d.fsize);
        ylabel('aceleration /ms^{-2}','fontsize',d.fsize);
    else
        %x vs y plot
        d.plot = plot( x,y,'r-' ); hold on
        d.plot_nodrag = plot( x_nodrag,y_nodrag,'b-' );
        d.plot_bound = plot( xbound,ybound,'m--' );
        xlabel('x/m','fontsize',d.fsize);
        ylabel('y/m','fontsize',d.fsize);
    end
    
    %Otherwise update existing parameters
elseif d.plot_and_save_PNG==0
    if get( d.h.POPUPMENUchoosegraph,'value' ) == 2
        %vx and vy vs t plot
        set( d.plot, 'xdata', t, 'ydata',vx );
        set( d.plot_nodrag, 'xdata', T, 'ydata',vx_nodrag );
        set( d.plot2, 'xdata', t, 'ydata',vy );
        set( d.plot2_nodrag, 'xdata', T, 'ydata',vy_nodrag );
    elseif get( d.h.POPUPMENUchoosegraph,'value' ) == 3
        %ax and ay vs t plot
        set( d.plot, 'xdata', t, 'ydata',ax );
        set( d.plot_nodrag, 'xdata', T, 'ydata',ax_nodrag );
        set( d.plot2, 'xdata', t, 'ydata',ay );
        set( d.plot2_nodrag, 'xdata', T, 'ydata',ay_nodrag );
    else
        %x vs y plot
        set( d.plot, 'xdata', x, 'ydata',y );
        set( d.plot_nodrag, 'xdata', x_nodrag, 'ydata',y_nodrag );
        set( d.plot_bound, 'xdata', xbound, 'ydata',ybound );
    end
end

%Set title, legend and filename
title({['Projectile with drag: u = ',num2str(d.u,d.sfs),'ms^{-1}, \theta = ',...
    num2str(d.theta_deg,d.sfs),'^o, h = ',num2str(d.y0,d.sfs),'m'],...
    ['\rho = ',num2str(d.rho,d.sfs),' kgm^{-3}, A = ',num2str(d.A,d.sfs),'m^2, m = ',...
    num2str(d.m,d.sfs),'kg, c_D = ',num2str(d.cD,d.sfs)]},'fontsize',d.fsize);
if get( d.h.POPUPMENUchoosegraph,'value' ) == 1
    d.L = legend({'drag','no drag'},'fontsize',d.fsize,'location','NorthEast');
    png_filename = 'projectile xy.png';
elseif get( d.h.POPUPMENUchoosegraph,'value' ) == 2
    d.L = legend({'vx','vy','vx no drag','vy no drag'},'fontsize',d.fsize,'location','NorthEast');
    png_filename = 'projectile vx vy.png';
elseif get( d.h.POPUPMENUchoosegraph,'value' ) == 3
    d.L = legend({'ax','ay','ax no drag','ay no drag'},'fontsize',d.fsize,'location','NorthEast');
    png_filename = 'projectile ax ay.png';
end

%Set axes scale, grid and fontsize
if d.fix_scale == 0
    axis tight
    if get( d.h.POPUPMENUchoosegraph,'value' ) == 1
        %x,y graph
        xlim( [0,d.xmax] );
        ylim( [0,d.ymax] );
    end
else
    axis manual
end
grid on
set(gca,'fontsize',d.fsize)

%Save externally plotted graph, if option set
if d.plot_and_save_PNG==1
    print( fig,png_filename,'-dpng','-r300');
    close(fig);
    figure(d.h.FIGUREprojectile); axes(d.h.AXESprojectile);
    d.plot_and_save_PNG=0;
    d.plot = [];
    d.plot_nodrag = [];
    d.plot2 = [];
    d.plot2_nodrag = [];
    d.L = [];
    projectile_update
end

%%

%figcenter
% Centres a figure with handle fig in the current display.

function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');
display_size=get(0,'MonitorPositions');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*( screen_size(3)-fig_position(3) ) - screen_size(1);
centre_y = 0.5*( screen_size(4)-fig_position(4) ) - screen_size(2);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%End of code






