%inclined_plane
% Projectile launched up an inclined plane
% A. French March 2020.
function inclined_plane

%Launch velocity /ms^-1
u = 10;

%Plane angle
theta = 30*pi/180;

%Strength of gravity /ms^-2
g = 9.81;

%Fontsize for plots
fsize = 18;

%

%Max r elevation
elev_max = pi/4 - theta/2;
rmax = ((u^2)/g)*( 1/ ( 1 + sin(theta) ) );

%Plot range vs elevation
[r,t,elev] = r_calc( theta, g, u, 1000 );
plot(elev*180/pi, r); hold on; set(gca, 'fontsize', fsize);
plot(elev_max*180/pi, rmax, 'r*' ); grid on;
xlabel('Elevation angle \epsilon /deg', 'fontsize', fsize);
ylabel('Range up slope /m', 'fontsize', fsize);
title(['Slope angle \theta=',num2str(theta*180/pi,3),'^o. Max range=',num2str(rmax,3),'m. ',...
    '\epsilon_{max}=',num2str(elev_max*180/pi,3),'^o.'], 'fontsize', fsize);
xlim([0,90]);
print(gcf, 'inclined plane r vs elev.png','-dpng','-r300' );

%Plot inclined plane
clf;
patch([0,rmax*cos(theta),rmax*cos(theta),0],...
    [0,0,rmax*sin(theta),0],'g','facealpha',0.5,'edgecolor','g');
hold on; grid on; set(gca, 'fontsize', fsize); axis equal

%Plot trajectory for various elevations
[r,t,elev] = r_calc( theta, g, u, 20 );
for n=1:length(elev);
    %x and y coordinates
    tt = linspace(0,t(n),1000);
    x = u*tt*cos(elev(n)+theta);
    y = u*tt*sin(elev(n)+theta) - 0.5*g*tt.^2;
    
    %Plot trajectory 
    plot(x,y);
end

%Axes labels
xlabel('x /m', 'fontsize', fsize);
ylabel('y /m', 'fontsize', fsize);
title(['Slope angle \theta=',num2str(theta*180/pi,3),'^o. Max range=',num2str(rmax,3),'m. ',...
    '\epsilon_{max}=',num2str(elev_max*180/pi,3),'^o.'], 'fontsize', fsize);

%Print and close
print( gcf,'inclined plane traj.png','-dpng','-r300' );
close(gcf);

%%

%Range, time of flight and elevation calculator
function [r,t,elev] = r_calc( theta, g, u, N )

%Elevation 
elev = linspace(0,pi/2 - theta, N);

%Time of flight /s
t = 2*u*sin(elev)/( g*cos(theta) );

%Range up slope /m
r = u*t.*cos(elev) - 0.5*g*sin(theta).*(t.^2);

%End of code
